# Copyright (c) 2024 Airbyte, Inc., all rights reserved.
"""Cloud destinations for Airbyte."""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

from airbyte_api.models import (
    DestinationBigquery,
    DestinationDuckdb,
    DestinationPostgres,
    DestinationSnowflake,
)

from airbyte.caches.base import CacheBase
from airbyte.caches.bigquery import BigQueryCache
from airbyte.caches.duckdb import DuckDBCache
from airbyte.caches.motherduck import MotherDuckCache
from airbyte.caches.postgres import PostgresCache
from airbyte.caches.snowflake import SnowflakeCache
from airbyte.exceptions import PyAirbyteSecretNotFoundError
from airbyte.secrets import get_secret
from airbyte.secrets.base import SecretString


if TYPE_CHECKING:
    from collections.abc import Callable

    from airbyte._util import api_util
    from airbyte.caches.base import CacheBase


SNOWFLAKE_PASSWORD_SECRET_NAME = "SNOWFLAKE_PASSWORD"


def destination_to_cache(
    destination_configuration: api_util.DestinationConfiguration | dict[str, Any],
) -> CacheBase:
    """Get the destination configuration from the cache."""
    conversion_fn_map: dict[str, Callable[[Any], CacheBase]] = {
        "bigquery": bigquery_destination_to_cache,
        "duckdb": duckdb_destination_to_cache,
        "motherduck": motherduck_destination_to_cache,
        "postgres": postgres_destination_to_cache,
        "snowflake": snowflake_destination_to_cache,
    }
    if isinstance(destination_configuration, dict):
        try:
            destination_type = (
                destination_configuration.get("DESTINATION_TYPE")
                or destination_configuration["destinationType"]
            )
            if hasattr(destination_type, "value"):
                destination_type = destination_type.value
            elif hasattr(destination_type, "_value_"):
                destination_type = destination_type._value_
            else:
                destination_type = str(destination_type)
        except KeyError as ex:
            raise ValueError(
                f"Missing 'destinationType' in keys {list(destination_configuration.keys())}."
            ) from ex
    else:
        destination_type = destination_configuration.DESTINATION_TYPE.value

    if destination_type not in conversion_fn_map:
        raise ValueError(
            "Cannot convert destination to a cache configuration. "
            f"Destination type {destination_type} not supported. "
            f"Supported cache types: {list(conversion_fn_map.keys())}"
        )

    conversion_fn = conversion_fn_map[destination_type]
    return conversion_fn(destination_configuration)


def bigquery_destination_to_cache(
    destination_configuration: DestinationBigquery | dict[str, Any],
) -> BigQueryCache:
    """Create a new BigQuery cache from the destination configuration.

    We may have to inject credentials, because they are obfuscated when config
    is returned from the REST API.
    """
    credentials_path = get_secret("BIGQUERY_CREDENTIALS_PATH")
    if isinstance(destination_configuration, dict):
        destination_configuration = DestinationBigquery(**destination_configuration)

    return BigQueryCache(
        project_name=destination_configuration.project_id,
        dataset_name=destination_configuration.dataset_id,
        credentials_path=credentials_path,
        dataset_location=destination_configuration.dataset_location,
    )


def duckdb_destination_to_cache(
    destination_configuration: DestinationDuckdb,
) -> DuckDBCache:
    """Create a new DuckDB cache from the destination configuration."""
    return DuckDBCache(
        db_path=destination_configuration.destination_path,
        schema_name=destination_configuration.schema or "main",
    )


def motherduck_destination_to_cache(
    destination_configuration: DestinationDuckdb,
) -> MotherDuckCache:
    """Create a new DuckDB cache from the destination configuration."""
    if not destination_configuration.motherduck_api_key:
        raise ValueError("MotherDuck API key is required for MotherDuck cache.")

    return MotherDuckCache(
        database=destination_configuration.destination_path,
        schema_name=destination_configuration.schema or "main",
        api_key=SecretString(destination_configuration.motherduck_api_key),
    )


def postgres_destination_to_cache(
    destination_configuration: DestinationPostgres,
) -> PostgresCache:
    """Create a new Postgres cache from the destination configuration."""
    port: int = int(destination_configuration.port) if destination_configuration.port else 5432
    if not destination_configuration.password:
        raise ValueError("Password is required for Postgres cache.")

    return PostgresCache(
        database=destination_configuration.database,
        host=destination_configuration.host,
        password=destination_configuration.password,
        port=port,
        schema_name=destination_configuration.schema or "public",
        username=destination_configuration.username,
    )


def snowflake_destination_to_cache(
    destination_configuration: DestinationSnowflake | dict[str, Any],
    password_secret_name: str = SNOWFLAKE_PASSWORD_SECRET_NAME,
) -> SnowflakeCache:
    """Create a new Snowflake cache from the destination configuration.

    We may have to inject credentials, because they are obfuscated when config
    is returned from the REST API.
    """
    if isinstance(destination_configuration, dict):
        destination_configuration = DestinationSnowflake(**destination_configuration)

    snowflake_password: str | None = None
    if (
        destination_configuration.credentials
        and hasattr(destination_configuration.credentials, "password")
        and isinstance(destination_configuration.credentials.password, str)
    ):
        destination_password = str(destination_configuration.credentials.password)
        if "****" in destination_password:
            try:
                snowflake_password = get_secret(password_secret_name)
            except ValueError as ex:
                raise PyAirbyteSecretNotFoundError(
                    "Password is required for Snowflake cache, but it was not available."
                ) from ex
        else:
            snowflake_password = get_secret(destination_password)
    else:
        snowflake_password = get_secret(password_secret_name)

    return SnowflakeCache(
        account=destination_configuration.host.split(".snowflakecomputing")[0],
        database=destination_configuration.database,
        schema_name=destination_configuration.schema,
        warehouse=destination_configuration.warehouse,
        role=destination_configuration.role,
        username=destination_configuration.username,
        password=snowflake_password,
    )



# Copyright (c) 2024 Airbyte, Inc., all rights reserved.
"""Cloud destinations for Airbyte."""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any

from airbyte_api.models import (
    BatchedStandardInserts,
    DatasetLocation,
    DestinationBigquery,
    DestinationDuckdb,
    DestinationPostgres,
    DestinationSnowflake,
    UsernameAndPassword,
)

from airbyte.secrets.base import SecretString


if TYPE_CHECKING:
    from collections.abc import Callable

    from airbyte._util import api_util
    from airbyte.caches.base import CacheBase
    from airbyte.caches.bigquery import BigQueryCache
    from airbyte.caches.duckdb import DuckDBCache
    from airbyte.caches.motherduck import MotherDuckCache
    from airbyte.caches.postgres import PostgresCache
    from airbyte.caches.snowflake import SnowflakeCache


SNOWFLAKE_PASSWORD_SECRET_NAME = "SNOWFLAKE_PASSWORD"


def cache_to_destination_configuration(
    cache: CacheBase,
) -> api_util.DestinationConfiguration:
    """Get the destination configuration from the cache."""
    conversion_fn_map: dict[str, Callable[[Any], api_util.DestinationConfiguration]] = {
        "BigQueryCache": bigquery_cache_to_destination_configuration,
        "bigquery": bigquery_cache_to_destination_configuration,
        "DuckDBCache": duckdb_cache_to_destination_configuration,
        "duckdb": duckdb_cache_to_destination_configuration,
        "MotherDuckCache": motherduck_cache_to_destination_configuration,
        "motherduck": motherduck_cache_to_destination_configuration,
        "PostgresCache": postgres_cache_to_destination_configuration,
        "postgres": postgres_cache_to_destination_configuration,
        "SnowflakeCache": snowflake_cache_to_destination_configuration,
        "snowflake": snowflake_cache_to_destination_configuration,
    }
    cache_class_name = cache.__class__.__name__
    if cache_class_name not in conversion_fn_map:
        raise ValueError(
            "Cannot convert cache type to destination configuration. "
            f"Cache type {cache_class_name} not supported. "
            f"Supported cache types: {list(conversion_fn_map.keys())}"
        )

    conversion_fn = conversion_fn_map[cache_class_name]
    return conversion_fn(cache)


def duckdb_cache_to_destination_configuration(
    cache: DuckDBCache,
) -> DestinationDuckdb:
    """Get the destination configuration from the DuckDB cache."""
    return DestinationDuckdb(
        destination_path=str(cache.db_path),
        schema=cache.schema_name,
    )


def motherduck_cache_to_destination_configuration(
    cache: MotherDuckCache,
) -> DestinationDuckdb:
    """Get the destination configuration from the DuckDB cache."""
    return DestinationDuckdb(
        destination_path=cache.db_path,
        schema=cache.schema_name,
        motherduck_api_key=cache.api_key,
    )


def postgres_cache_to_destination_configuration(
    cache: PostgresCache,
) -> DestinationPostgres:
    """Get the destination configuration from the Postgres cache."""
    return DestinationPostgres(
        database=cache.database,
        host=cache.host,
        password=cache.password,
        port=cache.port,
        schema=cache.schema_name,
        username=cache.username,
    )


def snowflake_cache_to_destination_configuration(
    cache: SnowflakeCache,
) -> DestinationSnowflake:
    """Get the destination configuration from the Snowflake cache."""
    return DestinationSnowflake(
        host=f"{cache.account}.snowflakecomputing.com",
        database=cache.get_database_name().upper(),
        schema=cache.schema_name.upper(),
        warehouse=cache.warehouse,
        role=cache.role,
        username=cache.username,
        credentials=UsernameAndPassword(
            password=cache.password,  # pyrefly: ignore[bad-argument-type]
        ),
    )


def bigquery_cache_to_destination_configuration(
    cache: BigQueryCache,
) -> DestinationBigquery:
    """Get the destination configuration from the BigQuery cache."""
    credentials_json: str | None = (
        SecretString(Path(cache.credentials_path).read_text(encoding="utf-8"))
        if cache.credentials_path
        else None
    )

    return DestinationBigquery(
        project_id=cache.project_name,
        dataset_id=cache.dataset_name,
        dataset_location=DatasetLocation(cache.dataset_location),
        credentials_json=credentials_json,
        loading_method=BatchedStandardInserts(),
    )



# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
"""Environment meta utils and globals.

This module contains functions for detecting environment and runtime information.
"""

from __future__ import annotations

import os
import shutil
import sys
from contextlib import suppress
from functools import lru_cache
from pathlib import Path
from platform import python_implementation, python_version, system

import requests

from airbyte.version import get_version


_MCP_MODE_ENABLED: bool = False
"""Whether we are running in MCP (Model Context Protocol) mode."""

COLAB_SESSION_URL = "http://172.28.0.12:9000/api/sessions"
"""URL to get the current Google Colab session information."""


def get_colab_release_version() -> str | None:
    if "COLAB_RELEASE_TAG" in os.environ:
        return os.environ["COLAB_RELEASE_TAG"]

    return None


def is_ci() -> bool:
    return "CI" in os.environ


def set_mcp_mode() -> None:
    """Set flag indicating we are running in MCP (Model Context Protocol) mode.

    This should be called early in MCP server initialization to ensure
    proper detection and prevent interactive prompts.
    """
    print(
        f"Running in MCP mode: PyAirbyte MCP v{get_version()} (Python v{python_version()})",
        file=sys.stderr,
    )
    global _MCP_MODE_ENABLED
    _MCP_MODE_ENABLED = True


def is_mcp_mode() -> bool:
    """Return True if running in MCP (Model Context Protocol) mode."""
    return _MCP_MODE_ENABLED


@lru_cache
def is_langchain() -> bool:
    """Return True if running in a Langchain environment.

    This checks for the presence of the 'langchain-airbyte' package.

    TODO: A more robust check would inspect the call stack or another flag to see if we are actually
          being invoked via LangChain, vs being installed side-by-side.

    This is cached for performance reasons.
    """
    return "langchain_airbyte" in sys.modules


def is_windows() -> bool:
    return system() == "Windows"


@lru_cache
def is_colab() -> bool:
    return bool(get_colab_release_version())


def is_interactive() -> bool:
    """Return True if running in an interactive environment where we can prompt users for input."""
    try:
        if is_ci() or is_mcp_mode():
            return False

        if is_colab() or is_jupyter():
            return True

        # No special modes detected. Return result based on whether stdin and stdout are ttys.
        return bool(
            sys.__stdin__ and sys.__stdin__.isatty() and sys.__stdout__ and sys.__stdout__.isatty()
        )
    except Exception:
        return False


@lru_cache
def is_jupyter() -> bool:
    """Return True if running in a Jupyter notebook or qtconsole.

    Will return False in Colab (use is_colab() instead).
    """
    try:
        shell = get_ipython().__class__.__name__  # type: ignore  # noqa: PGH003
    except NameError:
        return False  # If 'get_ipython' undefined, we're probably in a standard Python interpreter.

    if shell == "ZMQInteractiveShell":
        return True  # Jupyter notebook or qtconsole.

    if shell == "TerminalInteractiveShell":
        return False  # Terminal running IPython

    return False  # Other type (?)


@lru_cache
def get_notebook_name() -> str | None:
    if is_colab():
        session_info: dict | None = None
        with suppress(Exception):
            response = requests.get(COLAB_SESSION_URL)
            if response.status_code == 200:  # noqa: PLR2004  # Magic number
                session_info = response.json()[0]

        if session_info and "name" in session_info:
            return session_info["name"]

    return None


@lru_cache
def get_vscode_notebook_name() -> str | None:
    with suppress(Exception):
        import IPython  # noqa: PLC0415  # pyrefly: ignore[missing-import]

        return Path(
            IPython.extract_module_locals()[1]["__vsc_ipynb_file__"],
        ).name

    return None


def is_vscode_notebook() -> bool:
    return get_vscode_notebook_name() is not None


@lru_cache
def get_python_script_name() -> str | None:
    script_name = None
    with suppress(Exception):
        script_name = sys.argv[0]  # When running a python script, this is the script name.

    if script_name:
        return Path(script_name).name

    return None


@lru_cache
def get_application_name() -> str | None:
    return get_notebook_name() or get_python_script_name() or get_vscode_notebook_name() or None


def get_python_version() -> str:
    return f"{python_version()} ({python_implementation()})"


def get_os() -> str:
    if is_colab():
        return f"Google Colab ({get_colab_release_version()})"

    return f"{system()}"


@lru_cache
def which(executable_name: str) -> Path | None:
    """Return the path to an executable which would be run if the given name were called.

    This function is a cross-platform wrapper for the `shutil.which()` function.
    """
    which_executable: str | None = None
    which_executable = shutil.which(executable_name)
    if not which_executable and is_windows():
        # Try with the .exe extension
        which_executable = shutil.which(f"{executable_name}.exe")

    return Path(which_executable) if which_executable else None


def is_docker_installed() -> bool:
    return bool(which("docker"))



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Internal utility functions for MCP secret loading."""

from __future__ import annotations

import os
from pathlib import Path

import dotenv

from airbyte._util.meta import is_interactive
from airbyte.secrets import (
    DotenvSecretManager,
    GoogleGSMSecretManager,
    SecretSourceEnum,
    register_secret_manager,
)
from airbyte.secrets.config import disable_secret_source
from airbyte.secrets.util import get_secret, is_secret_available


AIRBYTE_MCP_DOTENV_PATH_ENVVAR = "AIRBYTE_MCP_ENV_FILE"


def _load_dotenv_file(dotenv_path: Path | str) -> None:
    """Load environment variables from a .env file."""
    if isinstance(dotenv_path, str):
        dotenv_path = Path(dotenv_path)
    if not dotenv_path.exists():
        raise FileNotFoundError(f".env file not found: {dotenv_path}")

    dotenv.load_dotenv(dotenv_path=dotenv_path)


def load_secrets_to_env_vars() -> None:
    """Load secrets from dotenv files and secret managers into environment variables.

    This function must be called before mcp_server() so that config args can resolve
    from the loaded environment variables.

    Note: Later secret manager registrations have higher priority than earlier ones.
    """
    # Load the .env file from the current working directory.
    envrc_path = Path.cwd() / ".envrc"
    if envrc_path.exists():
        envrc_secret_mgr = DotenvSecretManager(envrc_path)
        _load_dotenv_file(envrc_path)
        register_secret_manager(
            envrc_secret_mgr,
        )

    if AIRBYTE_MCP_DOTENV_PATH_ENVVAR in os.environ:
        dotenv_path = Path(os.environ[AIRBYTE_MCP_DOTENV_PATH_ENVVAR]).absolute()
        custom_dotenv_secret_mgr = DotenvSecretManager(dotenv_path)
        _load_dotenv_file(dotenv_path)
        register_secret_manager(
            custom_dotenv_secret_mgr,
        )

    if is_secret_available("GCP_GSM_CREDENTIALS") and is_secret_available("GCP_GSM_PROJECT_ID"):
        # Initialize the GoogleGSMSecretManager if the credentials and project are set.
        register_secret_manager(
            GoogleGSMSecretManager(
                project=get_secret("GCP_GSM_PROJECT_ID"),
                credentials_json=get_secret("GCP_GSM_CREDENTIALS"),
            )
        )

    # Make sure we disable the prompt source in non-interactive environments.
    if not is_interactive():
        disable_secret_source(SecretSourceEnum.PROMPT)



# Copyright (c) 2024 Airbyte, Inc., all rights reserved.
"""Useful constants for working with Airbyte Cloud features in PyAirbyte."""

from __future__ import annotations

from airbyte._util.api_imports import JobStatusEnum


FINAL_STATUSES: set[JobStatusEnum] = {
    JobStatusEnum.SUCCEEDED,
    JobStatusEnum.FAILED,
    JobStatusEnum.CANCELLED,
}
"""The set of `.JobStatusEnum` strings that indicate a sync job has completed."""

FAILED_STATUSES: set[JobStatusEnum] = {
    JobStatusEnum.FAILED,
    JobStatusEnum.CANCELLED,
}
"""The set of `.JobStatusEnum` strings that indicate a sync job has failed."""

READABLE_DESTINATION_TYPES: set[str] = {
    "bigquery",
    "snowflake",
}
"""List of Airbyte Cloud destinations that PyAirbyte is able to read from."""



# Copyright (c) 2024 Airbyte, Inc., all rights reserved.
"""Entry point for the Smoke Test source."""

import sys

from airbyte_cdk.entrypoint import launch

from airbyte.cli.smoke_test_source.source import SourceSmokeTest


def run() -> None:
    """Run the smoke test source."""
    launch(SourceSmokeTest(), sys.argv[1:])


if __name__ == "__main__":
    run()



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#
from __future__ import annotations

from setuptools import setup

setup(
    name="airbyte-source-test",
    version="0.0.1",
    description="Test Source",
    author="Airbyte",
    author_email="contact@airbyte.io",
    packages=["source_test"],
    entry_points={
        "console_scripts": [
            "source-test=source_test.run:run",
        ],
    },
)



# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
"""A simple test of PyAirbyte, using the Faker source connector.

Usage (from PyAirbyte root directory):
> poetry run python ./examples/run_github_samples.py

No setup is needed, but you may need to delete the .venv-source-faker folder
if your installation gets interrupted or corrupted.
"""

from __future__ import annotations

import airbyte as ab


# Create a token here: https://github.com/settings/tokens
GITHUB_TOKEN = ab.get_secret("GITHUB_PERSONAL_ACCESS_TOKEN")

FAILING_STREAMS = [
    "collaborators",
    "issue_timeline_events",  # key error: 'converted_to_discussion'
    "projects_v2",
    "team_members",
    "team_memberships",
    "teams",
]

source = ab.get_source(
    "source-github",
    config={
        "repositories": ["airbytehq/airbyte-lib-private-beta"],
        "credentials": {"personal_access_token": GITHUB_TOKEN},
    },
    streams="*",
)

streams = list(set(source.get_available_streams()) - set(FAILING_STREAMS))

source.print_samples(streams=streams)
